@@ -1,5 +1,6 @@
 
 #include <iostream>
+
 #include <pybind11/pybind11.h>
 #include <pybind11/embed.h>
 #include <pybind11/stl.h>
@@ -21,6 +22,9 @@
 #include <opencv2/imgcodecs/legacy/constants_c.h>
 #include "opencv2/core/types_c.h"
 
+#include "libmaix_cv_image.h"
+#include "libmaix_disp.h"
+#include "any.hpp"
 // using namespace cv;
 using namespace std;
 
@@ -28,70 +32,494 @@ namespace py = pybind11;
 #define heigh_t 10
 #define debug_line printf("%s:%d %s %s %s \r\n", __FILE__, __LINE__, __FUNCTION__, __DATE__, __TIME__)
 
-// typedef enum
-// {
-//     LAB = 0,
-//     INVALID ,
-//     BINARY,
-//     GRAY  ,
-//     RGB888,          // supported
-//     RGB565,
-//     RGBA8888,
-//     YUV420SP_NV21,   // supported
 
-// }image_mode_t;
 
-// struct libmaix_image
-// {
 
-//   cv::Mat obj;
 
-//   int _load() {
-
-//     return 0;
-//   }
-
-//   int _save() {
-
-//     return 0;
-//   }
-
-// };
-
-// class _maix_image :public libmaix_image
-// pybind11::class_<_maix_image>(m, "Image")
-//     .def(pybind11::init<>())
-//     .def("load", &_maix_image::test)
-//     .def("save", &_maix_image::test)
-//     .def("format", &_maix_image::test)
-//     .def("size", &_maix_image::test)
-//     .def("tobytes", &_maix_image::test)
-//     .def("resize", &_maix_image::test)
-//     .def("rotate", &_maix_image::test)
-//     .def("crop", &_maix_image::test)
-//     .def("convert", &_maix_image::test)
-//     .def("mode", &_maix_image::test)
-//     .def("draw_ellipse", &_maix_image::test)
-//     .def("draw_string", &_maix_image::test)
-//     .def("draw_circle", &_maix_image::test)
-//     .def("draw_rectangle", &_maix_image::test)
-//     .def("draw_line", &_maix_image::test)
-//     .def("load_freetype", &_maix_image::test);
 class _maix_image
 {
+
+private:
+  libmaix_image_t *img;
+    int _maix_image_img_num;
+  any maix_pram1[4];
+  any maix_pram2[4];
+  any opencv_parm1[4];
+
+
+  int get_to(string &mode)
+  {
+    if (mode == "L")
+    {
+      return 0;
+    }
+    else if (mode == "RGB16")
+    {
+      return 1;
+    }
+    else if (mode == "RGB")
+    {
+      return 2;
+    }
+    else if (mode == "RGBA")
+    {
+      return 3;
+    }
+    else
+    {
+      PyErr_SetString(PyExc_RuntimeError, "libmaix_disp_create err!");
+      throw py::error_already_set();
+      return -1;
+    }
+  }
+
 public:
+  std::string _maix_image_format;
+  std::string _maix_image_mode;
+  std::vector<int> _maix_image_size;
+  py::dict info;
+
   py::bytes test(py::bytes &rgb)
   {
     //   puts("test _maix_image");
-  }
-  _maix_image()
-  {
-    //   puts("new _maix_image");
+    return py::none();
   }
 
+  _maix_image &_new(vector<int> size, vector<int> color, string image_mode)
+  {
+    this->~_maix_image();
+    this->_maix_image_img_num = this->get_to(image_mode);
+    this->img = libmaix_image_create(size[0], size[1], any_cast<libmaix_image_mode_t>(maix_pram1[this->_maix_image_img_num]), LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+    libmaix_cv_image_draw_rectangle(this->img, 0, 0, size[0], size[1], MaixColor(color[0], color[1], color[2]), -1);
+    this->_maix_image_mode = image_mode;
+    this->_maix_image_size[0] = this->img->width;
+    this->_maix_image_size[1] = this->img->height;
+    this->_maix_image_format = image_mode;
+    return *this;
+  }
+
+  void _show()
+  {
+    if(NULL == this->img)
+    {
+      return;
+    }
+      struct libmaix_disp *disp = libmaix_disp_create(0);
+      if(disp)
+      {
+        libmaix_image_t *rgb888 = libmaix_image_create(disp->width, disp->height, LIBMAIX_IMAGE_MODE_RGB888, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        libmaix_cv_image_draw_rectangle(rgb888, 0, 0, 240, 240, MaixColor(0, 0, 0), -1);
+        libmaix_cv_image_draw_image(this->img, 0, 0, rgb888);
+
+        disp->draw_image(disp, rgb888);
+        libmaix_image_destroy(&rgb888);
+        
+      }
+      libmaix_disp_destroy(&disp);
+  }
+
+
+  _maix_image &load(py::object data, vector<int> size, string image_mode)
+  {
+    if (py::isinstance<py::bytes>(data))
+    {
+      this->~_maix_image();
+      this->_maix_image_img_num = this->get_to(image_mode);
+      this->img = libmaix_image_create(size[0], size[1], any_cast<libmaix_image_mode_t>(maix_pram1[this->_maix_image_img_num]), LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+      if (this->img)
+      {
+        string tmp = data.cast<string>();
+        memcpy(this->img->data, tmp.c_str(), this->img->width * this->img->height * any_cast<int>(maix_pram2[this->_maix_image_img_num]));
+      }
+      this->_maix_image_mode = image_mode;
+      this->_maix_image_size[0] = this->img->width;
+      this->_maix_image_size[1] = this->img->height;
+      this->_maix_image_format = image_mode;
+
+      return *this;
+
+      /*
+      if (image_mode == "RGB")
+      {
+        this->img = libmaix_image_create(size[0], size[1], LIBMAIX_IMAGE_MODE_RGB888, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        if (this->img)
+        {
+          string tmp = data.cast<string>();
+          memcpy(this->img->data, tmp.c_str(), this->img->width * this->img->height * 3);
+        }
+        this->mode = "RGB";
+        this->size[0] = this->img->width;
+        this->size[1] = this->img->height;
+        this->format = "RGB";
+
+        return *this;
+      }
+      else if (image_mode == "RGBA")
+      {
+        this->img = libmaix_image_create(size[0], size[1], LIBMAIX_IMAGE_MODE_RGBA8888, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        if (this->img)
+        {
+          string tmp = data.cast<string>();
+          memcpy(this->img->data, tmp.c_str(), this->img->width * this->img->height * 4);
+        }
+        this->mode = "RGBA";
+        this->size[0] = this->img->width;
+        this->size[1] = this->img->height;
+        this->format = "RGBA";
+        return *this;
+      }
+      else if (image_mode == "RGB16")
+      {
+        this->img = libmaix_image_create(size[0], size[1], LIBMAIX_IMAGE_MODE_RGB565, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        if (this->img)
+        {
+          string tmp = data.cast<string>();
+          memcpy(this->img->data, tmp.c_str(), this->img->width * this->img->height * 2);
+        }
+        this->mode = "RGB16";
+        this->size[0] = this->img->width;
+        this->size[1] = this->img->height;
+        this->format = "RGB16";
+        return *this;
+      }
+      else if (image_mode == "L")
+      {
+        if (size[0] == 0 || size[1] == 0)
+        {
+          size[0] = 240;
+          size[1] = 240;
+        }
+        this->img = libmaix_image_create(size[0], size[1], LIBMAIX_IMAGE_MODE_GRAY, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        if (this->img)
+        {
+          string tmp = data.cast<string>();
+          memcpy(this->img->data, tmp.c_str(), this->img->width * this->img->height);
+        }
+        this->mode = "L";
+        this->size[0] = this->img->width;
+        this->size[1] = this->img->height;
+        this->format = "L";
+        return *this;
+      }
+      */
+    }
+    return *this;
+  }
+  int save(string path)
+  {
+    if (this->img)
+    {
+      libmaix_cv_image_draw_image_save(this->img, path.c_str());
+    }
+    return 0;
+  }
+
+  py::bytes tobytes()
+  {
+    if (this->img)
+    {
+
+      return py::bytes((const char *)this->img->data, this->img->width * this->img->height * any_cast<int>(maix_pram2[this->_maix_image_img_num]));
+
+      /*
+      switch (this->img->mode)
+      {
+      case LIBMAIX_IMAGE_MODE_RGB888:
+        return py::bytes((const char *)this->img->data, this->img->width * this->img->height * 3);
+        break;
+      case LIBMAIX_IMAGE_MODE_RGBA8888:
+        return py::bytes((const char *)this->img->data, this->img->width * this->img->height * 4);
+        break;
+      case LIBMAIX_IMAGE_MODE_GRAY:
+        return py::bytes((const char *)this->img->data, this->img->width * this->img->height * 1);
+        break;
+      case LIBMAIX_IMAGE_MODE_RGB565:
+        return py::bytes((const char *)this->img->data, this->img->width * this->img->height * 2);
+        break;
+      default:
+        break;
+      }
+      */
+    }
+    return py::none();
+  }
+
+  _maix_image &resize(int w, int h)
+  {
+    if (this->img)
+    {
+      libmaix_image_t *tmp = libmaix_image_create(w, h, this->img->mode, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+      if (tmp)
+      {
+
+        cv::Mat src(this->img->width, this->img->height, any_cast<int>(opencv_parm1[this->_maix_image_img_num]), this->img->data);
+        cv::Mat dst(w, h, CV_8UC3, tmp->data);
+        cv::resize(src, dst, cv::Size(w, h));
+        libmaix_image_destroy(&this->img), this->img = tmp;
+        this->_maix_image_size[0] = w;
+        this->_maix_image_size[1] = h;
+
+        /*
+        switch (int(this->img->mode))
+        {
+        case (3): //LIBMAIX_IMAGE_MODE_RGB888
+        {
+          cv::Mat src(this->img->width, this->img->height, CV_8UC3, this->img->data);
+          cv::Mat dst(w, h, CV_8UC3, tmp->data);
+          cv::resize(src, dst, cv::Size(w, h));
+          libmaix_image_destroy(&this->img), this->img = tmp;
+        }
+
+        break;
+        case (4): //LIBMAIX_IMAGE_MODE_RGB565
+        {
+          cv::Mat src(this->img->width, this->img->height, CV_8UC2, this->img->data);
+          cv::Mat dst(w, h, CV_8UC2, tmp->data);
+          cv::resize(src, dst, cv::Size(w, h));
+          libmaix_image_destroy(&this->img), this->img = tmp;
+        }
+
+        break;
+        case (5): //LIBMAIX_IMAGE_MODE_RGBA8888
+        {
+          cv::Mat src(this->img->width, this->img->height, CV_8UC4, this->img->data);
+          cv::Mat dst(w, h, CV_8UC4, tmp->data);
+          cv::resize(src, dst, cv::Size(w, h));
+          libmaix_image_destroy(&this->img), this->img = tmp;
+        }
+
+        break;
+        case (2): //LIBMAIX_IMAGE_MODE_GRAY
+        {
+          cv::Mat src(this->img->width, this->img->height, CV_8UC1, this->img->data);
+          cv::Mat dst(w, h, CV_8UC1, tmp->data);
+          cv::resize(src, dst, cv::Size(w, h));
+          libmaix_image_destroy(&this->img), this->img = tmp;
+        }
+
+        break;
+        default:
+          break;
+        }
+
+        */
+      }
+    }
+    return *this;
+  }
+
+  _maix_image &rotate(double rotate)
+  {
+    if (this->img)
+    {
+      libmaix_image_t *tmp = libmaix_image_create(this->img->width, this->img->height, this->img->mode, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+      libmaix_cv_image_rotate(this->img, rotate, &tmp);
+      libmaix_image_destroy(&this->img), this->img = tmp;
+    }
+    return *this;
+  }
+
+  _maix_image &draw_crop(vector<int> thr)
+  {
+
+    if (this->img)
+    {
+      libmaix_image_t *tmp = libmaix_image_create(thr[2], thr[3], this->img->mode, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+      libmaix_cv_image_crop(this->img, thr[0], thr[1], thr[2], thr[3], &tmp);
+      libmaix_image_destroy(&this->img), this->img = tmp;
+      this->_maix_image_size[0] = thr[2];
+      this->_maix_image_size[1] = thr[3];
+    }
+    return *this;
+  }
+
+  _maix_image &draw_convert(string mode)
+  {
+    if (this->img)
+    {
+      int mode_tmp = this->get_to(mode);
+      libmaix_image_t *tmp = libmaix_image_create(this->img->width, this->img->height, any_cast<libmaix_image_mode_t>(maix_pram1[mode_tmp]), LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+      libmaix_cv_image_convert(this->img, any_cast<libmaix_image_mode_t>(maix_pram1[mode_tmp]), &tmp);
+      libmaix_image_destroy(&this->img), this->img = tmp;
+      this->_maix_image_mode = mode;
+
+      /*
+      if (mode == "RGB")
+      {
+        libmaix_image_t *tmp = libmaix_image_create(this->img->width, this->img->height, LIBMAIX_IMAGE_MODE_RGB888, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        libmaix_cv_image_convert(this->img, LIBMAIX_IMAGE_MODE_RGB888, &tmp);
+        libmaix_image_destroy(&this->img), this->img = tmp;
+      }
+      else if (mode == "RGBA")
+      {
+        libmaix_image_t *tmp = libmaix_image_create(this->img->width, this->img->height, LIBMAIX_IMAGE_MODE_RGBA8888, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        libmaix_cv_image_convert(this->img, LIBMAIX_IMAGE_MODE_RGBA8888, &tmp);
+        libmaix_image_destroy(&this->img), this->img = tmp;
+      }
+      else if (mode == "RGB16")
+      {
+        libmaix_image_t *tmp = libmaix_image_create(this->img->width, this->img->height, LIBMAIX_IMAGE_MODE_RGB565, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        libmaix_cv_image_convert(this->img, LIBMAIX_IMAGE_MODE_RGB565, &tmp);
+        libmaix_image_destroy(&this->img), this->img = tmp;
+      }
+      else if (mode == "L")
+      {
+        libmaix_image_t *tmp = libmaix_image_create(this->img->width, this->img->height, LIBMAIX_IMAGE_MODE_GRAY, LIBMAIX_IMAGE_LAYOUT_HWC, NULL, true);
+        libmaix_cv_image_convert(this->img, LIBMAIX_IMAGE_MODE_GRAY, &tmp);
+        libmaix_image_destroy(&this->img), this->img = tmp;
+      }
+      */
+    }
+    return *this;
+  }
+
+  _maix_image &draw_ellipse(vector<int> &rect, double angle, double startAngle, double endAngle, vector<int> color, int thickness)
+  {
+    if (this->img)
+    {
+      libmaix_cv_image_draw_ellipse(this->img, rect[0], rect[1], rect[2], rect[3], angle, startAngle, endAngle, MaixColor(color[0], color[1], color[2]), thickness);
+    }
+    return *this;
+  }
+  _maix_image &draw_string(int x, int y, const char *str, double scale, vector<int> color, int thickness)
+  {
+    if (this->img)
+    {
+      libmaix_cv_image_draw_string(this->img, x, y, str, MaixColor(color[0], color[1], color[2]), scale, thickness);
+    }
+    return *this;
+  }
+
+  _maix_image &draw_circle(vector<int> &circ, vector<int> &color, int thickness)
+  {
+    if (this->img)
+    {
+      libmaix_cv_image_draw_circle(this->img, circ[0], circ[1], circ[2], MaixColor(color[0], color[1], color[2]), thickness);
+    }
+    return *this;
+  }
+
+  _maix_image &draw_rectangle(vector<int> &rect, vector<int> color, int thickness)
+  {
+    if (img)
+    {
+      libmaix_cv_image_draw_rectangle(img, rect[0], rect[1], rect[2], rect[3], MaixColor(color[0], color[1], color[2]), thickness);
+    }
+    return *this;
+  }
+
+  _maix_image &draw_line(vector<int> &lin, vector<int> color, int thickness)
+  {
+    if (img)
+    {
+      libmaix_cv_image_draw_line(this->img, lin[0], lin[1], lin[2], lin[3], MaixColor(color[0], color[1], color[2]), thickness);
+    }
+    return *this;
+  }
+
+  // _maix_image &load_freetype(string &path)
+  // {
+
+  //     libmaix_cv_image_load_freetype(const char *path);
+
+  // }
+
+  _maix_image()
+  {
+    img = NULL;
+    maix_pram1[0] = LIBMAIX_IMAGE_MODE_GRAY;
+    maix_pram2[0] = 1;
+    opencv_parm1[0] = CV_8UC1;
+
+    maix_pram1[1] = LIBMAIX_IMAGE_MODE_RGB565;
+    maix_pram2[1] = 2;
+    opencv_parm1[1] = CV_8UC2;
+
+    maix_pram1[2] = LIBMAIX_IMAGE_MODE_RGB888;
+    maix_pram2[2] = 3;
+    opencv_parm1[2] = CV_8UC3;
+
+    maix_pram1[3] = LIBMAIX_IMAGE_MODE_RGBA8888;
+    maix_pram2[3] = 4;
+    opencv_parm1[3] = CV_8UC4;
+
+    this->_maix_image_size.push_back(0);
+    this->_maix_image_size.push_back(0);
+  }
   ~_maix_image()
   {
-    //   puts("del _maix_image");
+    if (img)
+    {
+      libmaix_image_destroy(&img);
+    }
+  }
+};
+
+
+
+
+
+
+
+_maix_image Image_new(vector<int> size, vector<int> color, string image_mode)
+{
+
+
+
+
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+class _maix_vision_histogram : public py::list
+{
+private:
+  /* data */
+public:
+  _maix_vision_histogram()
+  {
+  }
+  _maix_vision_histogram(py::list hist)
+  {
+    this->append(hist);
+  }
+  ~_maix_vision_histogram()
+  {
+  }
+  py::object bins()
+  {
+  }
+  py::object l_bins()
+  {
+  }
+  py::object a_bins()
+  {
+  }
+  py::object b_bins()
+  {
+  }
+  py::object get_percentile(float percentile)
+  {
+  }
+  py::object get_threshold()
+  {
+  }
+  py::object get_statistics()
+  {
   }
 };
 
@@ -172,6 +600,7 @@ private:
         }
       }
     }
+    return py::none();
   }
   //==================================================================
   //函数名：  py_img_to_in_img
@@ -322,10 +751,6 @@ public:
     int size = input.total() * input.elemSize();
     return py::bytes((char *)input.data, size);
   }
-  // medianBlur   //中值滤波
-  // GaussianBlur //高斯滤波
-  // Canny        //Canny 检测边缘
-  // HoughCircles //霍夫圆变换原理及圆检测
 
   //==================================================================
   //函数名：  _maix_vision_medianBlur
@@ -367,12 +792,12 @@ public:
   //          返回高斯滤波后的图像,图像格式和输入保持一致；
   //修改记录：
   //==================================================================
-  py::object _maix_vision_GaussianBlur(py::object py_img, int ksize_w, int ksize_h, double sigmaX, double sigmaY, int borderType, std::vector<int> size, int mode)
+  py::object _maix_vision_GaussianBlur(py::object py_img, std::vector<int> ksize, double sigmaX, double sigmaY, int borderType, std::vector<int> size, int mode)
   {
     cv::Mat in_img;
     py_img_to_in_img(py_img, in_img, size, mode); //获取图像
     cv::Mat dist;
-    cv::GaussianBlur(in_img, dist, cv::Size(ksize_w, ksize_h), sigmaX, sigmaY);
+    cv::GaussianBlur(in_img, dist, cv::Size(ksize[0], ksize[1]), sigmaX, sigmaY);
     py::object tmp = out_img_to_py_img(py_img, dist);
     return tmp;
   }
@@ -438,6 +863,58 @@ public:
     }
     return return_val;
   }
+  //==================================================================
+  //函数名：  _maix_vision_opencv_calcHist
+  //作者：    dianjixz
+  //日期：    2021-10-29
+  //功能：    opencv计算图像直方图
+  //输入参数：
+  //          py::object py_img     python输入图像对象
+  //          int channels          需要统计直方图的第几通道(默认为0)
+  //          vector<int> &roi      掩膜，，计算掩膜内的直方图  感兴趣区域(默认全部区域)
+  //          int histSize          直方图分成多少个区间，就是 bin的个数(默认256)
+  //          vector<int> ranges     统计像素值得区间(默认(0,256))
+  //          bool uniform          是否对得到的直方图数组进行归一化处理(默认true)
+  //          bool accumulate       在多个图像时，是否累计计算像素值得个数(默认false)
+  //          vector<int> size      图像的尺寸(非必须)(默认(0,0))
+  //          int mode              图像的格式(非必须)(默认16)
+  //返回值：
+  //          返回中值滤波后的图像,图像格式和输入保持一致；
+  //修改记录：
+  //==================================================================
+  py::object _maix_vision_opencv_calcHist(py::object py_img, int channels, vector<int> &roi, int histSize, vector<int> ranges, bool uniform, bool accumulate, vector<int> size, int mode)
+  {
+    py::list return_val;
+    cv::Mat in_img;
+    this->py_img_to_in_img(py_img, in_img, size, mode); //获取图像
+    cv::Rect rect;
+    if (roi[2] != 0 && roi[3] != 0)
+    {
+      rect.x = roi[0];
+      rect.y = roi[1];
+      rect.width = roi[2];
+      rect.height = roi[3];
+    }
+    else
+    {
+      rect.x = 0;
+      rect.y = 0;
+      rect.width = in_img.size[0];
+      rect.height = in_img.size[1];
+    }
+    cv::Mat mask = cv::Mat::zeros(in_img.size(), CV_8UC1);
+    mask(rect).setTo(255);
+    float range[] = {ranges[0], ranges[1]};
+    const float *histRanges = {range};
+    cv::Mat _hist;
+    calcHist(&in_img, 1, (const int *)&channels, mask, _hist, 1, (const int *)&histSize, &histRanges, uniform, accumulate);
+    for (int i = 0; i < histSize; i++)
+    {
+      return_val.append(_hist.at<float>(i));
+    }
+    return return_val;
+  }
+
   //==================================================================
   //函数名：  get_blob_color_max
   //作者：    dianjixz
@@ -1032,8 +1509,63 @@ public:
     return_val["rotation"] = k;
     return std::move(return_val);
   }
+  //==================================================================
+  //函数名：  _maix_vision_get_histogram
+  //作者：    dianjixz
+  //日期：    2021-10-29
+  //功能：    在 roi 的所有颜色通道上进行标准化直方图运算，并返回 histogram 对象
+  //输入参数：
+  //          py::object py_img                        python输入图像对象
+  //          vector<vector<int>> &thresholds          thresholds 必须是元组列表。 [(lo, hi), (lo, hi), ..., (lo, hi)] 定义你想追踪的颜色范围,对于灰度图像，每个元组需要包含两个值 - 最小灰度值和最大灰度值。
+  //          bool invert                              反转阈值操作，像素在已知颜色范围之外进行匹配，而非在已知颜色范围内。
+  //          vector<int> &roi                         是感兴趣区域的矩形元组(x，y，w，h)。如果未指定，ROI即整个图像的图像矩形。操作范围仅限于 roi 区域内的像素。
+  //          int bins                                 和其他bin是用于直方图通道的箱数。对于灰度图像，使用 bins
+  //          vector<int> size      图像的尺寸(非必须)(默认(0,0))
+  //          int mode              图像的格式(非必须)(默认16)
+  //返回值：
+  //          返回中值滤波后的图像,图像格式和输入保持一致；
+  //修改记录：
+  //==================================================================
+  py::object _maix_vision_get_histogram(py::object py_img, vector<vector<int>> &thresholds, bool invert, vector<int> &roi, int bins, vector<int> size, int mode)
+  {
+    py::list return_val;
+    cv::Mat in_img;
+    this->py_img_to_in_img(py_img, in_img, size, mode); //获取图像
+    cv::Rect rect;
+    if (roi[2] != 0 && roi[3] != 0)
+    {
+      rect.x = roi[0];
+      rect.y = roi[1];
+      rect.width = roi[2];
+      rect.height = roi[3];
+    }
+    else
+    {
+      rect.x = 0;
+      rect.y = 0;
+      rect.width = in_img.size[0];
+      rect.height = in_img.size[1];
+    }
+    cv::Mat mask = cv::Mat::zeros(in_img.size(), CV_8UC1);
+    mask(rect).setTo(255);
+    // vector<float> ranges1;
+
+    // float range[] = {ranges[0], ranges[1]};
+    // const float *histRanges = {range};
+    // cv::Mat _hist;
+    // calcHist(&in_img, 1, (const int*)&channels, mask, _hist, 1, (const int*)&histSize, &histRanges, uniform, accumulate);
+    // for(int i=0;i<histSize;i++)
+    // {
+    //   return_val.append(_hist.at<float>(i));
+    // }
+    // return return_val;
+  }
 };
 
+
+
+
+
 PYBIND11_MODULE(_maix_opencv, m)
 {
   pybind11::class_<_maix_vision>(m, "Vision")
@@ -1043,45 +1575,49 @@ PYBIND11_MODULE(_maix_opencv, m)
       .def_readonly("COLOR_RGBA", &_maix_vision::COLOR_RGBA)
       .def_readonly("COLOR_L", &_maix_vision::COLOR_L)
       //opencv原生函数
-      .def("opecv_medianBlur", &_maix_vision::_maix_vision_medianBlur, py::arg("py_img"), py::arg("m_size") = 5, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("opecv_GaussianBlur", &_maix_vision::_maix_vision_GaussianBlur, py::arg("py_img"), py::arg("ksize_w"), py::arg("ksize_h"), py::arg("sigmaX"), py::arg("sigmaY"), py::arg("borderType"), py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("opecv_Canny", &_maix_vision::_maix_vision_Canny, py::arg("py_img"), py::arg("thr_h"), py::arg("thr_l"), py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("opecv_HoughCircles", &_maix_vision::_maix_vision_HoughCircles, py::arg("py_img"), py::arg("method") = 3, py::arg("dp"), py::arg("minDist"), py::arg("param1"), py::arg("param2"), py::arg("minRadius"), py::arg("maxRadius"), py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
+      .def("opecv_medianBlur", &_maix_vision::_maix_vision_medianBlur, py::arg("py_img"), py::arg("m_size") = 5, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("opecv_GaussianBlur", &_maix_vision::_maix_vision_GaussianBlur, py::arg("py_img"), py::arg("ksize"), py::arg("sigmaX"), py::arg("sigmaY"), py::arg("borderType"), py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("opecv_Canny", &_maix_vision::_maix_vision_Canny, py::arg("py_img"), py::arg("thr_h"), py::arg("thr_l"), py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("opecv_HoughCircles", &_maix_vision::_maix_vision_HoughCircles, py::arg("py_img"), py::arg("method") = 3, py::arg("dp"), py::arg("minDist"), py::arg("param1"), py::arg("param2"), py::arg("minRadius"), py::arg("maxRadius"), py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
       //基于opencv编写MaixPy3特有函数
-      .def("get_blob_lab", &_maix_vision::get_blob_color_max, py::arg("py_img"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("critical") = 0, py::arg("color") = 0, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("get_blob_color", &_maix_vision::get_blob_color_max, py::arg("py_img"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("critical") = 0, py::arg("color") = 0, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("find_blob_lab", &_maix_vision::_maix_vision_find_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("x_stride") = 2, py::arg("y_stride") = 2, py::arg("invert") = 0, py::arg("area_threshold") = 10, py::arg("pixels_threshold") = 10, py::arg("merge") = 0, py::arg("margin") = 0, py::arg("tilt") = 0, py::arg("co") = 1, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("find_ball_lab", &_maix_vision::_maix_vision_find_ball_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("co") = 1, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("find_circles_blob", &_maix_vision::_maix_vision_find_ball_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("co") = 1, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
-      .def("find_line", &_maix_vision::find_line, py::arg("py_img"), py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
+      .def("get_blob_lab", &_maix_vision::get_blob_color_max, py::arg("py_img"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("critical") = 0, py::arg("color") = 0, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("get_blob_color", &_maix_vision::get_blob_color_max, py::arg("py_img"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("critical") = 0, py::arg("color") = 0, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("find_blob_lab", &_maix_vision::_maix_vision_find_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("x_stride") = 2, py::arg("y_stride") = 2, py::arg("invert") = 0, py::arg("area_threshold") = 10, py::arg("pixels_threshold") = 10, py::arg("merge") = 0, py::arg("margin") = 0, py::arg("tilt") = 0, py::arg("co") = 1, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("find_ball_lab", &_maix_vision::_maix_vision_find_ball_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("co") = 1, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("find_circles_blob", &_maix_vision::_maix_vision_find_ball_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("co") = 1, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
+      .def("find_line", &_maix_vision::find_line, py::arg("py_img"), py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
       //基于opencv编写兼容openmv图像处理函数
       .def("get_histogram", &_maix_vision::test)
       .def("get_percentile", &_maix_vision::test)
       .def("get_threshold", &_maix_vision::test)
       .def("get_statistics", &_maix_vision::test)
-      .def("find_blobs", &_maix_vision::_maix_vision_find_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("x_stride") = 2, py::arg("y_stride") = 2, py::arg("invert") = 0, py::arg("area_threshold") = 10, py::arg("pixels_threshold") = 10, py::arg("merge") = 0, py::arg("margin") = 0, py::arg("tilt") = 0, py::arg("co") = 1, py::arg("size") = std::vector<int>{0, 0}, py::arg("mode") = 16)
+      .def("find_blobs", &_maix_vision::_maix_vision_find_blob, py::arg("py_img"), py::arg("thresholds"), py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("x_stride") = 2, py::arg("y_stride") = 2, py::arg("invert") = 0, py::arg("area_threshold") = 10, py::arg("pixels_threshold") = 10, py::arg("merge") = 0, py::arg("margin") = 0, py::arg("tilt") = 0, py::arg("co") = 1, py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = 16)
       .def("find_lines", &_maix_vision::test)
       .def("find_line_segments", &_maix_vision::test)
       .def("get_regression", &_maix_vision::test)
       .def("find_circles", &_maix_vision::test)
       .def("find_rects", &_maix_vision::test);
 
-  pybind11::class_<_maix_image>(m, "Image")
-      .def(pybind11::init<>())
-      .def("load", &_maix_image::test)
-      .def("save", &_maix_image::test)
-      .def("format", &_maix_image::test)
-      .def("size", &_maix_image::test)
-      .def("tobytes", &_maix_image::test)
-      .def("resize", &_maix_image::test)
-      .def("rotate", &_maix_image::test)
-      .def("crop", &_maix_image::test)
-      .def("convert", &_maix_image::test)
-      .def("mode", &_maix_image::test)
-      .def("draw_ellipse", &_maix_image::test)
-      .def("draw_string", &_maix_image::test)
-      .def("draw_circle", &_maix_image::test)
-      .def("draw_rectangle", &_maix_image::test)
-      .def("draw_line", &_maix_image::test)
-      .def("load_freetype", &_maix_image::test);
-}
+  // pybind11::class_<_maix_image>(m, "Image")
+  //     .def(pybind11::init<>())
+  //     .def_readonly("format", &_maix_image::_maix_image_format)
+  //     .def_readonly("size", &_maix_image::_maix_image_size)
+  //     .def_readonly("mode", &_maix_image::_maix_image_mode)
+  //     .def("new", &_maix_image::_new, py::arg("size") = std::vector<int>{240, 240}, py::arg("color") = std::vector<int>{0, 0, 0}, py::arg("mode") = "RGB")
+  //     .def("show", &_maix_image::_show)
+  //     .def("load", &_maix_image::load, py::arg("data"), py::arg("size") = std::vector<int>{240, 240}, py::arg("mode") = "RGB")
+  //     .def("save", &_maix_image::save, py::arg("path"))
+  //     .def("tobytes", &_maix_image::tobytes)
+  //     .def("resize", &_maix_image::resize, py::arg("w"), py::arg("h"))
+  //     .def("rotate", &_maix_image::rotate, py::arg("rotate"))
+  //     .def("crop", &_maix_image::draw_crop, py::arg("thr"))
+  //     .def("convert", &_maix_image::draw_convert, py::arg("mode") = "RGB")
+  //     .def("draw_ellipse", &_maix_image::draw_ellipse, py::arg("rect"), py::arg("angle"), py::arg("startAngle"), py::arg("endAngle"), py::arg("color"), py::arg("thickness"))
+  //     .def("draw_string", &_maix_image::draw_string, py::arg("x"), py::arg("y"), py::arg("str"), py::arg("scale") = 1.0, py::arg("color") = std::vector<int>{127, 127, 127}, py::arg("thickness") = 1)
+  //     .def("draw_circle", &_maix_image::draw_circle, py::arg("circ"), py::arg("color") = std::vector<int>{127, 127, 127}, py::arg("thickness") = 1)
+  //     .def("draw_rectangle", &_maix_image::draw_rectangle, py::arg("rect"), py::arg("color") = std::vector<int>{127, 127, 127}, py::arg("thickness") = 1)
+  //     .def("draw_line", &_maix_image::draw_line, py::arg("line"), py::arg("color") = std::vector<int>{127, 127, 127}, py::arg("thickness") = 1)
+  //     .def("load_freetype", &_maix_image::test);
+
+
+}
\ No newline at end of file
